/*
	wipos.h
	<c> Risto Holopainen 2021
	
	wavetable interpolating oscillators
	... but you probably can do this in a better way!
	

	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef WIPOS_H
#define WIPOS_H


#include "../util/oscil.h"
#include "../util/waveforms.h"



class wipos	// timbre interpolating oscil
{
	public:
		enum WAVETYPE {PRIME, SAW, PINK, TRI, PULSE};
		wipos(int sr, WAVETYPE WT);
		~wipos() { delete oa, delete ob; delete oc; delete[] wta; delete[] wtb; delete[] wtc; }
		double gen(double frq);
	
	private:
		int K, L;
		int sr;
		double q, b;
		double *wta, *wtb, *wtc;
		oscil *oa, *ob, *oc;
		double x, y, z, s;
};

wipos::wipos(int fs, WAVETYPE TT)
{
	sr = fs;
	L=16385, K=4097;
	wta = new double[L];
	wtb = new double[L];
	wtc = new double[K];
	waveform::make_triangle(wtc, K, 3);
	
	switch (TT)
		{
		case (PRIME) :
			{
			waveform::make_prime(wta, L, 12);
			waveform::make_prime(wtb, L, 5);
			break;
			}
		case (SAW) :
			{
			waveform::make_sawtooth(wta, L, 63);
			waveform::make_sawtooth(wtb, L, 11);
			break;
			}
		case (PINK) :
			{
			waveform::make_pink(wta, L, 63);
			waveform::make_pink(wtb, L, 11);
			break;
			}
		case (PULSE) :
			{
			waveform::make_pulse(wta, L, 1/8., 63);
			waveform::make_pulse(wtb, L, 1/8., 11);	
			break;
			}
		case (TRI) :
			{
			waveform::make_triangle(wta, L, 33);
			waveform::make_triangle(wtb, L, 6);
			break;				
			}
		default :
			{
			waveform::make_order3(wta, L, 63);
			waveform::make_order3(wtb, L, 11);
			}
		}

	oa = new oscil(sr, wta, L);
	ob = new oscil(sr, wtb, L);
	oc = new oscil(sr, wtc, K);
	q = 0;
	b = 0;
	s = 0;
	x = y = z = 0;
}

double wipos::gen(double f)
{
	double F = fabs(f);
	x = oa->gen(f);
	y = ob->gen(f);
	z = oc->gen(f);
	
	if(F < 500)
		{
		b = F/500.;
		s = (1-b)*x + b*y;
		}

	else if(F >= 500 && F < 1000)
		s = y;

	else if(F >= 1000 && F < 2400)	// f > 1kHz
		{
		q = (f-1000) / 1400.;
		s = (1-q)*y + q*z;
		}

	else // if(F >= 4000)
		s = z;

	return s;
}





#endif // define WIPOS_H
