/*
	parabolic oscillator
	Copyright Risto Holopainen 2021
	
	 This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.	
*/

#ifndef PARABOLIC_H
#define PARABOLIC_H

#define SR 48000	// global samplerate


typedef struct osc
{
	double phi;	// phase in -1..+1
	double x0;
	double x1;
	unsigned int sr;

} osc;


void init_osc(osc *Os, unsigned int fs)
{
	Os->phi = 0.0;
	Os->x0 = Os->x1 = 0.0;
	Os->sr = fs;
}


double parabolic_tri(double frq, osc *X)
{
	double omega = frq / X->sr;
	X->phi += omega;
	while(X->phi > 1.0)
		X->phi -= 2.0;
	while(X->phi < -1.0)
		X->phi += 2.0;

	double v = X->phi;
	double s = (v > 0) ? v : -v;
	double u = v*(1.0 - s);

	X->x1 = X->x0;
	X->x0 = u;

	return (X->x0 - X->x1) / omega;
}






#endif


