/*
	constants.h
	<c> Risto Holopainen 2021
	 
	frequencies for quasiperiodic tuning operations
	continued fractions etc

	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
*/

#include <cmath>


namespace constants
{

double cfrac(const unsigned int *q, int n)
{
	// const int n == q[n] --- size;
	int i = n;
	double x = 1;

	while(--i >= 0)
		x = 1.0/x + q[i];

	return 1.0/x;
}



const unsigned int primes[] = {2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61};

const unsigned int cf_f[] = {1,2,6,24,120,720,5040,40320,362880,3628800};

const unsigned int cf_A[] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30};

const unsigned int cf_G[] = {1,4,9,16,25,36,49,64,81,100,121,144,169,196,225};

const unsigned int cf_F[] = {1,1,2,3,5,8,13,21,34,55,89,144,233,377,610};

const unsigned int cf_O[] = {1,3,5,7,9,11,13,15,17,19,21,23,25,27,29,31,33,35,37,39,41,43,45,47,49};

const unsigned int cf_D[] = {1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768};

const unsigned int cf_T[] = {1,3,9,27,81,243,729,2187,6561,19683,59049};

const unsigned int cf_d[] = {1,3,7,15,31,63,127,255,511,1023,2047,4095};

const unsigned int cf_R[] = {2,5,8,11,14,17,20,23,26,29,32,35,38,41,44,47,50,53,56,59};

// odd, 2^n, 3^n etc, fibo, ...

const double _A = cfrac(cf_A, 30);
const double _f = cfrac(cf_f, 10);
const double _G = cfrac(cf_G, 15);
const double _P = cfrac(primes, 18);
const double _F = cfrac(cf_F, 15);
const double _O = cfrac(cf_O, 25);
const double _D = cfrac(cf_D, 16);
const double _T = cfrac(cf_T, 11);
const double _d = cfrac(cf_d, 12);
const double _R = cfrac(cf_R, 20);
const double phi = (sqrt(5)+1)/2.;

const double PI = acos(-1);

double plastic()
{
	// calculate the plastic number
	// x^3 = x + 1
	const double s = sqrt(69);
	const double rho = cbrt( (9+s)/18. ) + cbrt( (9-s)/18. );
	return rho;
}


const double magic_numbers[31] = {sqrt(2), sqrt(3), sqrt(5), sqrt(7), sqrt(11), sqrt(13),
								phi, 3/2., 5/2., PI, 5/3., 7/3., plastic(),
								7/5., 11/5., exp(1), 11/7., 13/7., 13/11.,
								_A, _f, _G, _P, _F, _O, _D, _T, _d, _R, log(2), PI*PI/6};



}	// end namespace constants
